/*
 * This file contains an experiment done in paper
 * P. Richtárik and M. Takáč
 *      Parallel Coordinate Descent Methods for Big Data Optimization
 * http://www.optimization-online.org/DB_HTML/2012/11/3688.html
 */




#include "../helpers/c_libs_headers.h"
#include "../helpers/gsl_random_helper.h"

#include "../solver/structures.h"
#include "../class/loss/losses.h"
#include "../problem_generator/generator_nesterov.h"
#include "mex.h"

template<typename L, typename D>
double run_computation(problem_data<L, D> &inst, std::vector<D> &h_Li,
		double fvalOpt, int omp_threads, L n, L m, L sigma, int N,
		int blockReduction, std::vector<gsl_rng *>& rs,
		ofstream& experimentLogFile, const int MAXIMUM_THREADS, double fvaldctdn, double &totalRunningTime, double *objvals, double *cpuTimes, L &itrct) {
        
    double temp;
    itrct = 1;
    ofstream myfile;
    ostringstream oss;
    oss << n;
    string filename = "pcdm";
    filename.append(oss.str());
    filename.append(".txt");
    myfile.open(filename.c_str());
    
    omp_set_num_threads(omp_threads);
    init_random_seeds(rs);
    for (L i = 0; i < n; i++)
	inst.x[i] = 0;
    std::vector<D> residuals(m);
    Losses<L, D, square_loss_traits>::recompute_residuals(inst, residuals);
    D fvalInit = Losses<L, D, square_loss_traits>::compute_fast_objective(inst,
			residuals);
    
    //totalRunningTime = 0;
    double iterations = 0;
    L perPartIterations = n / blockReduction;
    double additional = perPartIterations / (0.0 + n);
    D fval = fvalInit;
    
    // store initial objective value
    experimentLogFile << setprecision(16) << omp_threads << "," << n << "," << m
		      << "," << sigma << "," << totalRunningTime << "," << iterations
		      << "," << fval << endl;
   
    double startTime;
	double endTime;
    
    //iterate
    for (int totalIt = 0; totalIt < N; totalIt++) {
	startTime = gettime_();
#pragma omp parallel for
	for (L it = 0; it < perPartIterations; it++) {
	    // one step of the algorithm
	    unsigned long int idx = gsl_rng_uniform_int(gsl_rng_r, n);
	    Losses<L, D, square_loss_traits>::do_single_iteration_parallel(inst, idx, residuals, inst.x, h_Li);
        }
	endTime = gettime_();
	iterations += additional;
	totalRunningTime += endTime - startTime;
	// recompute residuals  - this step is not necessary but if accumulation of rounding errors occurs it is useful
	omp_set_num_threads(MAXIMUM_THREADS);
	if (totalIt % 3 == 0) {
	    Losses<L, D, square_loss_traits>::recompute_residuals(inst, residuals);
	}
	fval = Losses<L, D, square_loss_traits>::compute_fast_objective(inst,residuals);
        
        objvals[itrct]  = fval;
        cpuTimes[itrct] = totalRunningTime;
        itrct++;
        
	int nnz = 0;
#pragma omp parallel for reduction(+:nnz)
	for (L i = 0; i < n; i++)
	    if (inst.x[i] != 0)
		nnz++;
	omp_set_num_threads(omp_threads);
	cout << omp_threads << "," << n << ","
	     << m << "," << sigma << "," << totalRunningTime << ","
	     << iterations << "," << fval << "," << fvalOpt << "," << nnz
             << endl;

	experimentLogFile << setprecision(16) << omp_threads << "," << n << ","
			  << m << "," << sigma << "," << totalRunningTime << ","
			  << iterations << "," << fval << "," << fvalOpt << "," << nnz
			  << endl;
        
        myfile << fval << endl;
        myfile << fvalOpt << endl;
        
        myfile << "Check termination" << endl;
        temp = fval-fvalOpt;
        if (temp <= fvaldctdn - fvalOpt) break;
    }
    
    cout << endl;
    cout << "Precision solved: " << temp << ", Iterations: "<< iterations << endl;
    myfile << "Precision solved: " << temp << ", Iterations: "<< iterations << endl;
    
    cout << "CPU time: " << totalRunningTime << endl;
    myfile << "CPU time: " << totalRunningTime << endl;
    myfile.close();
    
    return fval;
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{    
    const int MAXIMUM_THREADS = mxGetScalar(prhs[5]);
    omp_set_num_threads(MAXIMUM_THREADS);

    mwIndex n = mxGetScalar(prhs[0]);
    mwIndex m = mxGetScalar(prhs[1]);
    mwIndex p = mxGetScalar(prhs[2]);
    
    double fvalOpt = mxGetScalar(prhs[3]);
    double fvaldctdn = mxGetScalar(prhs[4]);
    double omega     = mxGetScalar(prhs[6]);
    mwIndex N         = mxGetScalar(prhs[9]);

    //---------------------- Set output files
    ofstream histogramLogFile;
    histogramLogFile.open("results/large_scale_expeiment_histogram.log");
    ofstream experimentLogFile;
    experimentLogFile.open("results/large_scale_expeiment.log");

    // setup GSL random generators
    gsl_rng_env_setup();
    const gsl_rng_type * T;
    gsl_rng * r;
    T = gsl_rng_default;
    r = gsl_rng_alloc(T);
    std::vector<gsl_rng *> rs(MAXIMUM_THREADS);
    for (mwIndex i = 0; i < MAXIMUM_THREADS; i++) {
 	rs[i] = gsl_rng_alloc(T);
	gsl_rng_set(rs[i], i);
    }
    init_omp_random_seeds();
    
    problem_data<mwIndex, double> inst;
    init_random_seeds(rs);
    inst.A_csc_col_ptr = mxGetJc(prhs[7]);
    inst.A_csc_row_idx = mxGetIr(prhs[7]);
    inst.A_csc_values  = mxGetPr(prhs[7]);
    inst.b  = mxGetPr(prhs[8]);
    inst.omega = omega;
    inst.n = n;
    inst.m = m;
    inst.x.resize(n);

    inst.lambda = 1;
    std::vector<double> h_Li(n, 0);
    
    /*** Extra Output ***/
    double *objvals  = new double[2*N];
    double *cpuTimes = new double[2*N];
    mwIndex itrct = 0;
    
    cpuTimes[itrct] = 1.0e-5;
    
    double n2b = 0;
    
    for (mwIndex j=0; j<m; j++)
    {
        n2b += inst.b[j]*inst.b[j];
    }
    n2b *= 0.5;
    
    objvals[itrct] = n2b;

    double totalRunningTime2;
    double startTime = gettime_();
    cout << "Running experiment with " << MAXIMUM_THREADS << " threads" << endl;
    inst.sigma = 1 + (MAXIMUM_THREADS - 1) * (inst.omega - 1) / (inst.n - 1);
    cout << setprecision(16) << "beta = " << inst.sigma << endl;
    Losses<mwIndex, double, square_loss_traits>::compute_reciprocal_lipschitz_constants(inst, h_Li);
    double endTime = gettime_();
    totalRunningTime2 = endTime - startTime;
    
    // run the experiment
    startTime = gettime_();
    double totalRunningTime;
    double fvalPCDM = run_computation<mwIndex, double>(inst, h_Li, fvalOpt, MAXIMUM_THREADS, n, m, 
                      inst.sigma, N, 1, rs, experimentLogFile, MAXIMUM_THREADS, fvaldctdn, totalRunningTime2, objvals, cpuTimes, itrct);
    endTime = gettime_();
    totalRunningTime = endTime - startTime;
    
    histogramLogFile.close();
    experimentLogFile.close();
    
    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    double *fvalPCDMout = mxGetPr(plhs[0]);
    *fvalPCDMout = fvalPCDM;
    
    double *pt_plhs1, *pt_plhs2, *pt_plhs3, *pt_plhs4, *pt_plhs5;
    plhs[1]  = mxCreateDoubleMatrix(1, 1, mxREAL);
    pt_plhs1 = mxGetPr(plhs[1]);
    pt_plhs1[0] = totalRunningTime;
    
    plhs[2]  = mxCreateDoubleMatrix(1, 1, mxREAL);
    pt_plhs2 = mxGetPr(plhs[2]);
    pt_plhs2[0] = totalRunningTime2;
    
    plhs[3]  = mxCreateDoubleMatrix(itrct, 1, mxREAL);
    pt_plhs3 = mxGetPr(plhs[3]);
    
    //#pragma omp parallel for num_threads(MAXIMUM_THREADS)
    for (mwIndex j=0; j<itrct; j++)
        pt_plhs3[j] = objvals[j];
    
    plhs[4]  = mxCreateDoubleMatrix(itrct, 1, mxREAL);
    pt_plhs4 = mxGetPr(plhs[4]);
    
    //#pragma omp parallel for num_threads(MAXIMUM_THREADS)
    for (mwIndex j=0; j<itrct; j++)
        pt_plhs4[j] = cpuTimes[j];
    
    plhs[5]  = mxCreateDoubleMatrix(inst.n, 1, mxREAL);
    pt_plhs5 = mxGetPr(plhs[5]);
    
    //#pragma omp parallel for num_threads(MAXIMUM_THREADS)
    for (mwIndex j=0; j<inst.n; j++)
        pt_plhs5[j] = inst.x[j];
}
