function load_logistic(name,cross_valid,save_name)
% This script loads logistic regression problems using LIBSVM
% and compares solvers pdNCG, FISTA and PCDM.

addpath(genpath('../scripts/'));
addpath(genpath('../pdNCGv2/'));
addpath(genpath('../liblinear-1.96/'));
addpath(genpath('../TFOCS-1.3.1'));
addpath(genpath('../pcdm_logistic/'));
    
[labels, inst] = libsvmread(name);
    
[m, n] = size(inst);

%% Start cross-validation.
if cross_valid   
    best_tau = 1.0e-2;
    tau_best_string = num2str(best_tau);
    best_acc = 0;
    tau_current = best_tau;
    tau_current_string = num2str(tau_current);
    options_liblinear = ['-s 6 -c ',tau_current_string,' -v 5'];

    for i=0:2
        model = train(labels, inst, options_liblinear);
        if model > best_acc
            best_acc = model;
            best_tau = tau_current;
            tau_best_string = num2str(best_tau);
        end
        tau_current = tau_current*50;
        tau_current_string = num2str(tau_current);
        options_liblinear = ['-s 6 -c ',tau_current_string,' -v 5'];
    end
else
    best_tau = 1;
    tau_best_string = num2str(best_tau);
end
%% Call pdNCG.
options.n        = n;
options.X        = inst;
options.y        = labels;
options.tau      = 1/best_tau;
options.problem  = 'lg';
options.mu       = 1.0e-4;
options.tol      = 1.0e-4;
options.maxpdNCGitr = 50;
options.mxitrpcg = 10;
options.reg      = 1.0e-1;
options.c3       = 0.5;
options.max_itr_bt = 1;
options.verbose  = 2;

tic;
[x_pdNCG, ~, out_pdNCG] = pdNCGinterface(options);
time_pdNCG = toc;    
    
%% Call the TFOCS (FISTA N83) solver
opts = struct('restart',-Inf,'tol',1e-4,'maxits',1500,'alg','N83');
opts.errFcn = @(f,primal) toc; 
    
tic;
[x_tfocs, out_tfocs] = tfocs( smooth_Logistic(inst,labels), [], prox_l1( 1/best_tau ), zeros(n,1), opts );
time_tfocs = toc;

%% Call pcdm
threads = 40;
omega = max(sum(inst~=0,2));
p = randperm(n);   
phi_logi = @(w) sum(log(1 + exp(-labels.*(inst*w))));
obj_pdNCG = (1/best_tau)*norm(x_pdNCG,1) + phi_logi(x_pdNCG);
                                                            
[fval_pcdm, cpptime_pcdm, cpptime_pcdm2, objvals_pcdm, cpuTimes_pcdm, x_pcdm] = large_scale_experiment(n, m, p, obj_pdNCG - 1.0e-1, obj_pdNCG, threads, omega, inst, full(labels), 800, best_tau);

%% Call Liblinear solver
%options_liblinear = ['-s 6 -e 1.0e-3 -c ',tau_best_string];

%tic;
%model = train(labels, inst, options_liblinear);
%time_liblinear = toc;
    
%% Print info.    
obj_tfocs = (1/best_tau)*norm(x_tfocs,1) + phi_logi(x_tfocs);
%obj_liblinear = (1/best_tau)*norm(model.w',1) + phi_logi(model.w');
obj_pcdm = (1/best_tau)*norm(x_pcdm,1) + phi_logi(x_pcdm);

fprintf('\n');
fprintf('Best tau by cross-validation is: %d\n', (1/best_tau));
%fprintf('liblinear, CPU time: %1.2e, Obj: %1.2e \n',time_liblinear,obj_liblinear);
fprintf('pdNCG, CPU time: %1.2e, Obj: %1.2e \n',time_pdNCG,obj_pdNCG);
fprintf('FISTA, CPU time: %1.2e, Obj: %1.2e \n',time_tfocs,obj_tfocs);
fprintf('PCDM, CPU time: %1.2e, Obj: %1.2e \n',cpptime_pcdm2,obj_pcdm);

figure(1);
loglog(out_pdNCG.t,out_pdNCG.f,'b','LineWidth',3);
hold on;
loglog(out_tfocs.err,out_tfocs.f,'r--','LineWidth',3);
hold on;
loglog(cpuTimes_pcdm,objvals_pcdm,'g-.','LineWidth',3);

leg = legend('pdNCG','FISTA','PCDM');
set(leg,'FontSize',13,'Location','NorthEast');

ylabel('\bf{f_\tau(x)}', 'FontSize', 16, 'Rotation', 90);
xlabel('\bf{wall clock time}', 'FontSize', 16);

save([save_name,'.mat']);

end
