function ls_generator(n,eig_lo,eig_up,save_name,pcdm_max_itr,tfocs_max_itr)
% This script generates a sparse Least-Squares instance (A,b,xopt) 
%
% A: is the linear operator in the least-squares fidelity term
% b: is the data
% xopt: is the optimal solution of the problem
%
% xopt:= argmin ||x||_1 + 0.5||Ax-b||_2^2
%
% and compares solvers pdNCG, FISTA and PCDM.

addpath(genpath('../pdNCGv2/'));
addpath(genpath('../TFOCS-1.3.1'));
addpath(genpath('../pcdm_least_squares/'));

randn('state',250);
rand ('state',250);

% number of samples.
m = 2*n;
    
% Sparsity of xopt.
k = n/2^7;

% Generate singular values for matrix A.
S = eig_up*rand(n,1);
S = S + eig_lo;

% This operation mimics the standar MATLAB command 
% R = sprand(m,n,density,rc), where rc := S and density is set such that 
% there are only two nonzeros per row.
theta = 2*pi/1000;
ct = cos(theta);
st = sin(theta);
R = [
     ct  -st
     st   ct
    ];
Rt = R';

Gt = kron(sparse(1:n/2,1:n/2,ones(n/2,1),n/2,n/2),sparse(Rt));

% Build matrix A.
A = [sparse(1:n,1:n,S,n,n)*Gt;sparse(m-n,n)];

% Build optimal solution xopt.
temp = Gt'*(100./(S.^2));
[~, idx] = sort(abs(temp));
xopt = zeros(n,1);
xopt(idx(1:k/2)) = temp(idx(1:k/2));
xopt(idx(end:-1:end-k/2 + 1)) = temp(idx(end:-1:end-k/2 + 1));

% Build noise.
sub_g = zeros(n,1);
sub_g(xopt ~= 0) = sign(xopt(xopt ~= 0));
sub_g(xopt == 0) = 1.8.*rand(sum(xopt == 0),1) - 0.9;

e_n = (Gt*sub_g)./S;
e_n_to_m = rand(m-n,1);

e = [e_n;e_n_to_m];

% Create b and b_n (noise version).
b = A*xopt;
b_n = b + e;

tau = 1.0;

[m,n] = size(A);

fvalOpt = tau*norm(xopt,1) + 0.5*norm(A*xopt-b_n)^2;
    
%% Call pdNCG.
options.n        = n;
options.A        = A;
options.b        = b_n;
options.tau      = tau;
options.problem  = 'ls';
options.mu       = 1.0e-6;
options.tol      = 1.0e-8;
options.maxpdNCGitr = 50;
options.verbose  = 2;

tic;
[x_pdNCG, ~, out_pdNCG] = pdNCGinterface(options);
time_pdNCG = toc;    

%% Call the TFOCS solver
opts = struct('restart',-Inf,'tol',1e-13,'maxits',tfocs_max_itr,'alg','N83');
opts.errFcn = @(f,primal) toc; 
fvalpdNCG = tau*norm(x_pdNCG,1) + 0.5*norm(A*x_pdNCG-b_n)^2;
opts.stopFcn = @(f,primal) f <= fvalpdNCG;
    
tic;
[x_tfocs, out_tfocs] = tfocs( smooth_quad, {A, -b_n}, prox_l1( tau ), [], opts );
time_tfocs = toc;

%% Call pcdm
   
threads = 40;
omega = max(sum(A~=0,2));
p = randperm(n);      
                                                            
[fval_pcdm, cpptime_pcdm, cpptime_pcdm2, objvals_pcdm, cpuTimes_pcdm, x_pcdm] = large_scale_experiment(n, m, p, fvalOpt, fvalpdNCG, threads, omega, A, full(b_n), pcdm_max_itr);

save('-v7.3',[save_name,'.mat']);

%% Print info.
phi_ls = @(w) 0.5*norm(A*w-b_n,2)^2;
    
obj_tfocs = tau*norm(x_tfocs,1) + phi_ls(x_tfocs);
obj_pdNCG = tau*norm(x_pdNCG,1) + phi_ls(x_pdNCG);
obj_pcdm = tau*norm(x_pcdm,1) + phi_ls(x_pcdm);
    
fprintf('\n');
fprintf('Best tau by cross-validation is: %d\n', tau);
fprintf('Optimal objective %1.2e \n',fvalOpt);
fprintf('pdNCG, CPU time: %1.2e, Obj: %1.2e \n',time_pdNCG,obj_pdNCG);
fprintf('tfocs, CPU time: %1.2e, Obj: %1.2e \n',time_tfocs,obj_tfocs);
fprintf('pcdm, CPU time: %1.2e, Obj: %1.2e \n',cpptime_pcdm2,obj_pcdm);

figure(1);
loglog(out_pdNCG.t,out_pdNCG.f,'b','LineWidth',3);
hold on;
loglog(out_tfocs.err,out_tfocs.f,'r--','LineWidth',3);
hold on;
loglog(cpuTimes_pcdm,objvals_pcdm,'g-.','LineWidth',3);

leg = legend('pdNCG','FISTA','PCDM');
set(leg,'FontSize',13,'Location','NorthEast');
ylabel('\bf{f_\tau(x)}', 'FontSize', 16, 'Rotation', 90);
xlabel('\bf{wall clock time (seconds)}', 'FontSize', 16);

fprintf('\n');
fprintf('cond(A^TA): %1.15e\n', (max(S)/min(S))^2);
fprintf('Sparsity A: %1.15e\n', nnz(A)/(n*m));
fprintf('Omega: %1.15e\n', full(omega));

%clear all;

